import java.util.Map;

/**
 * An interface representing a HuffmanTree, used for CS261 Hwk 6, Fa 2014.
 * Includes methods for building the tree from a frequency map, adding new entries, and accessing the bitstrings of entries.
 * 
 * @author Joel
 * @version Fa 2014
 */
public interface AbstractHuffmanTree<E> extends AbstractBinaryTree<E>
{
	/**
	 * Rebuilds this HuffmanTree from the given list of item frequencies.
	 * Note that this method is destructive--it will override any previous elements in the tree.
	 * (Hint: let this method be a helper you call from the appropriate constructor!)
	 * @param frequencies A list of Frequency objects representing the elements and the frequency of their occurrence.
	 */
	public void createFromFrequencies(Map<E, Integer> frequencies);
	
	/**
	 * Adds an element with the given bit string code to the HuffmanTree. Note that the "weights" of nodes will not be maintained with this method.
	 * @param element The element to add to the tree
	 * @param bitString A bit string (e.g., "1101") to associate with that element and indicate its place in the tree. If the code is already in use, then it is replaced.
	 * @throws IllegalArgumentException if the bitString is not valid
	 */
	public void add(E element, String bitString);

	/**
	 * Returns a String representation of the tree's "code list"--that is, a list of elements (in String representation) and their bit string codes.
	 * The String should be a comma separated list of elements and their codes, on a single line. Elements and codes are separated by colons.
	 * For example, a HuffmanTree261<Character> might return "l:000,s:001,o:01,g:10, :110,e:1110,r:1111"
	 * @return A String list of the tree's elements and their associated bit codes
	 */
	public String getCodeList();

	/**
	 * Returns the tree's bit string representation of the given item. This allows you to look up the code for a particular element
	 * @param e The element to look up the code of
	 * @return A bit string corresponding to that element, or null if the element is not in the tree.
	 */
	public String getBitString(E e);

	
}
