import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.TreeMap;

/**
 * A class that is able to encode and decode text files using Huffman encoding.
 * 
 * @author YOUR_NAME_HERE
 */
public class HuffmanFileCodec
{
	public static final char EOT = '\u0003'; //a special, non-printable character that marks the end of the encoded text
	private AbstractHuffmanTree<Character> huffTree;
	
	/**
	 * Constructs a new codec object
	 */
	public HuffmanFileCodec()
	{
		huffTree = null;
	}

	
	/**
	 * Creates an encoding tree using the frequencies of the letters in the given input file
	 * @param input A file used to form the basis of the Huffman encoding
	 */
	public void createTree(File input)
	{
		TreeMap<Character, Integer> frequencies = new TreeMap<Character, Integer>(); //a map of frequencies

		try
		{
			FileReader reader = new FileReader(input);
			int v = reader.read(); //read one character (as an int)
			while(v != -1)
			{
				char c = (char)v; //turn into char
				
				//increase the frequency of that character
				Integer freq = frequencies.get(c);
				if(freq == null)
					freq = 0;
				frequencies.put(c,freq+1);
				
				v = reader.read();
			}
			
			reader.close();
		}
		catch(IOException e){
			e.printStackTrace();
			System.exit(1);
		}
		frequencies.put(EOT, 1); //end-of-text character (appears once). Not an actual character in the file, but we're pretending
		
		huffTree = new HuffmanTree<Character>(frequencies); //create the HuffmanTree
	}

	
	/**
	 * Creates an encoding tree using the frequencies of the letters in the given String representation of an encoding
	 * The String should be a comma separated list of elements and their codes, on a single line. Elements and codes are separated by colons.
	 * This String should have a format such as: "l:000,s:001,o:01,g:10, :110,e:1110,r:1111"
	 * that is, the same format generated by the AbstractHuffmanTree<Character>.getCodeList() method
	 * @param codelist The list of Huffman codes, in String format
	 */
	public void createTree(String codelist)
	{
		huffTree = new HuffmanTree<Character>(); //new empty tree

		String[] codes = codelist.split("(?<=\\d),"); //regex magic to split on commas that only follow numbers (so aren't the ',' element)
		
		for(int i=0; i<codes.length; i++) //go through the codes
		{
			char c = codes[i].charAt(0); //first char is the symbol
			String bitString = codes[i].substring(2); //cut off symbol and : to get the bit string
			huffTree.add(c, bitString); //add the symbol and its bit string to the tree
		}
	}

	
	/**
	 * Encodes the given input file using the Huffman encoding of the current tree, writing that file to disk. The method returns a reference to this new file.
	 * The encoded file will be created in the same directory, with the same file name but with the extension ".encoded" instead of the original extension.
	 * @param input The file to encode
	 * @return a reference to the new, encoded file.
	 */
	public File encode(File input)
	{
		//TODO Fill me in!
		
		return null;
	}
	
	
	/**
	 * Decodes the given encoded file, decoding it into the given PrintStream for output
	 * The encoded file must a String of the huffman codes as the first line, and end with an EOT character
	 * @param encodedFile The encoded file to decode
	 * @param out A PrintStream to output to. This can either be System.out, or a File. In either case, the method will "print()" the decoded file there. 
	 */
	public void decode(File encodedFile, PrintStream out)
	{
		//TODO Fill me in!
		
	}

	
	/**
	 * Returns a String representation of the codec's current encoding function (e.g., of the Huffman tree)
	 * @return A String showing the encoding function of the current Huffman Tree
	 */
	public String getEncoding()
	{
		if(huffTree == null)
			throw new IllegalStateException("Huffman tree has not been created; no encoding exists");
		
		return huffTree.getCodeList();
	}
}
